#include <sys/types.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#define CHUNK_SIZE 256*512

uint8_t group_start_code[4]             ={0x0, 0x0, 0x1, 0xB8};

static int verbose;

int verifycode(uint8_t * source, size_t bytes, uint8_t * match)
{
        if (bytes==0) return 0;

        if (!memcmp(source,match,bytes)) return 1;
        return 0;
}

/* Find offset of any GOP in a video chunk, and print it out */
int find_GOP(uint8_t *buf, size_t num,int ofs)
{
 size_t i, total;

 total= num - sizeof(group_start_code);

 for (i=0;i<total;i++,buf++,ofs++)
  if (verifycode(buf, sizeof(group_start_code), group_start_code)) {
    printf("Found GOP at %d\n",ofs); return(ofs);
  }
  return(0);
}

void print_somehex(uint8_t *cptr, int num)
{
 int i;
 int total;

 total=16; if (num<total) total=num;

 for (i=0;i<total;i++,cptr++) {
   printf("%02x ",(*cptr & 0xff));
 }
}

static void
parse_chunk(uint8_t buf[CHUNK_SIZE], int chunknum)
{
  uint8_t type, rec_type[256];
  unsigned int ofs, rec_start[256];
  unsigned int size, rec_size[256];
  uint8_t num_recs = buf[0];
  int i;
  uint8_t *p;
  int total_audio = 0;
  int total_video = 0;
  int first_vidoffset=0;
  int overall_total;
  int unknown_recs = 0;
  int vid16=0;
  int aud16=0;

  printf("Chunk %d has %d records\n",chunknum, num_recs);
  /*
   * each 128k chunk starts with N 16 byte records that tell you what sorts
   * of things are in the chunk
   */
  p = &buf[4];
  ofs = 4 + num_recs * 16;

  for (i = 0; i < num_recs; i++) {
    rec_start[i]=ofs;
    size = (p[0] << 8 | p[1]) << 4 | (p[2] >> 4); rec_size[i]=size;
    type = p[3]; rec_type[i]=type;

    printf("  #%02d size %05d ofs %05d", i, size, ofs);

    switch(type) {
      case 0xe0:
        printf(" vid: ");
        if (first_vidoffset==0) first_vidoffset=ofs;
	if ((size==16) && (vid16==0)) vid16=ofs;
        if (ofs<CHUNK_SIZE) {
	  print_somehex(&buf[ofs],size);
          printf("\n");
        }
        total_video += size;
	break;
      case 0xc0:
        printf(" aud: ");
        if (ofs<CHUNK_SIZE) print_somehex(&buf[ofs],size);
	if ((size==16) && (aud16==0)) aud16=ofs;
        printf("\n");
        total_audio += size;
	break;
      default:
        printf(" unknown\n");
        unknown_recs++;
    }
    p += 16; ofs+= size;
  }

  if (first_vidoffset!=0) {
    if (aud16) printf("Found aud16 at %d\n",aud16);
    ofs= find_GOP(&buf[first_vidoffset], total_video, first_vidoffset);
    if (vid16) printf("Found vid16 at %d\n",vid16);
    if (ofs && vid16) printf("GOT - vid16 is %d\n", ofs - vid16);
    for (i = 0; i < num_recs; i++) {
	if ((rec_size[i]==16) && (ofs>=rec_start[i]) &&
		(ofs < (rec_start[i] + rec_size[i])) )
		printf("Naughty GOP in a 16-byte record\n");
    }
  }

  overall_total= total_audio + total_video + num_recs * 16 + 4;
  printf("  total video: 0x%x total audio: 0x%x total: 0x%x\n",
        total_video, total_audio, overall_total);
  printf("  total video: %d total audio: %d total: %d diff %d\n\n",
        total_video, total_audio, overall_total, CHUNK_SIZE - overall_total);
}

static void
usage(void)
{
  printf("vsplit [options] <infile> <videofile> <audiofile>\n");
  printf("options:\n");
  printf("  -v                 verbose\n");
  printf("  -h                 help\n");
  exit(1);
}

int
main(int argc, char *argv[])
{
  char *fname;
  int fd, chunk;
  uint8_t buf[CHUNK_SIZE];
  extern char *optarg;
  extern int optind;
  int c;

  while ((c = getopt(argc, argv, "vh")) != -1) {
    switch (c) {
    case 'v': verbose++; break;
    case 'h':
    default: usage();
    }
  }

  argc -= optind;
  argv += optind;

  if (argc < 1) usage();
  fname = argv[0];
  fd = open(fname, O_RDONLY);


  /* discard the first chunk - it often contains garbage */
  read(fd, buf, CHUNK_SIZE); chunk=1;

  while (read(fd, buf, CHUNK_SIZE) == CHUNK_SIZE) {
    parse_chunk(buf,chunk); chunk++;
  }
  return(0);
}
